// Copyright (C) 2004, Brian Enigma <enigma at netninja.com>
// This file is part of iGallery.
//
// iGallery is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// iGallery is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Foobar; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
package org.ninjasoft.igallery.logic;

import java.util.*;
import java.io.*;
import javax.xml.parsers.*;
import org.ninjasoft.igallery.data.*;

/**
 * This is where all the magic happens.
 * @author enigma
 */
public class IGalleryLogic {
    private String username = "";
    private String password = "";
    private String url = "";
    private GalleryConnection galleryConnection = null;
    private StatusListener listener = null;
    private Vector galleryAlbums = new Vector();
    private Vector iPhotoAlbums = new Vector();
    private Hashtable iPhotoPictures = new Hashtable();
    private Vector missingLocal = new Vector();
    private Vector missingRemote = new Vector();
    
    public IGalleryLogic(StatusListener listener) {
        this.listener = listener;
    }
    
    public boolean loadIPhotoAlbums() {
        sendStatus("Loading iPhoto album list");
        this.iPhotoAlbums.removeAllElements();
        this.iPhotoPictures.clear();
        AlbumSaxHandler handler = new AlbumSaxHandler();
        try{
            SAXParserFactory.newInstance().newSAXParser().parse(
                    new FileInputStream(System.getProperty("user.home") + "/Pictures/iPhoto Library/AlbumData.xml"), 
                    handler);
        }catch(Throwable t){
            t.printStackTrace();
            sendError(t.toString());
            return false;
        }
        sendStatus("Obtained data about " + handler.getAlbums().size() + " album(s), " + handler.getPictureHash().size() + " picture(s)");
        this.iPhotoAlbums = handler.getAlbums();
        this.iPhotoPictures = handler.getPictureHash();
        return true;
    }
    
    public boolean login(String url, String username, String password) {
        sendStatus("Connecting");
        try{
        	this.galleryConnection = new GalleryConnection(url, listener);
        	this.galleryConnection.login(username, password);
        }catch(Throwable t) {
            t.printStackTrace();
            sendError(t.toString());
            this.galleryConnection = null;
            return false;
        }
        try{
            sendStatus("Fetching list of albums");
        	Hashtable albumList = galleryConnection.getAlbumList();
            galleryAlbums.removeAllElements();
            galleryAlbums.addAll(albumList.values());
        }catch(Throwable t){
            t.printStackTrace();
            sendError(t.toString());
            return false;
        }
        return true;
    }
    
    public boolean getAlbumDetails() {
    	for (Iterator i = this.galleryAlbums.iterator(); i.hasNext(); ) {
            GalleryAlbum album = (GalleryAlbum) i.next();
            try{
                sendStatus("Getting photo list for " + album.getName());
            	Hashtable photos = galleryConnection.getPhotoList(album);
                album.addPictures(photos.values());
            }catch(Throwable t){
                t.printStackTrace();
                sendError(t.toString());
                return false;
            }
        }
        sendStatus("Finished getting album details");
        return true;
    }
    
    public void compareAlbums(IPhotoAlbumData iPhotoAlbum, GalleryAlbum galleryAlbum) {
        Vector iPhotoPictures = iPhotoAlbum.getPictures();
        Vector galleryPictures = galleryAlbum.getPictures();
        missingLocal.removeAllElements();
        missingRemote.removeAllElements();
        // Not the most efficient way of doing this...sort of like a Bubble Sort...
        // Check if local pics reside remotely
        for (Iterator i = iPhotoPictures.iterator(); i.hasNext(); ) {
            boolean found = false;
            Integer index = (Integer) i.next();
            IPhotoPictureData iPhotoPicture = (IPhotoPictureData) this.iPhotoPictures.get(index);
            String name = new File(iPhotoPicture.getFilename()).getName();
            for (Iterator j = galleryPictures.iterator(); j.hasNext(); ) {
                GalleryImage galleryPicture = (GalleryImage) j.next();
                if (name.equalsIgnoreCase(galleryPicture.getName())) {
                    found = true;
                    break;
                }
            }
            if (!found)
                missingRemote.add(iPhotoPicture);
        }
        // Check if remote pics reside locally
        for (Iterator i = galleryPictures.iterator(); i.hasNext(); ) {
            boolean found = false;
            GalleryImage galleryPicture = (GalleryImage) i.next();
            for (Iterator j = iPhotoPictures.iterator(); j.hasNext(); ) {
                Integer index = (Integer) j.next();
                IPhotoPictureData iPhotoPicture = (IPhotoPictureData) this.iPhotoPictures.get(index);
                String name = new File(iPhotoPicture.getFilename()).getName();
                if (galleryPicture.getName().equalsIgnoreCase(name)) {
                    found = true;
                    break;
                }
            }
            if (!found)
                missingLocal.add(galleryPicture);
        }
    }
    
    public void logout() {
        this.galleryConnection = null;
        this.galleryAlbums.removeAllElements();
    }
    
    public boolean upload(GalleryAlbum album, IPhotoPictureData picture) {
        try{
        	File file = new File(picture.getFilename());
        	this.galleryConnection.uploadPhoto(album, file, file.getName(), picture.getCaption());
            return true;
        }catch(Throwable t){
            t.printStackTrace();
            sendError(t.toString());
            return false;
        }
    }
    
    private void sendStatus(String message) {
        if (listener != null)
            listener.setStatus(message);
    }
    private void sendError(String message) {
        if (listener != null)
            listener.setError(message);
    }
    
    public Vector getGalleryAlbums()        {return this.galleryAlbums;}
    public Vector getIPhotoAlbums()         {return this.iPhotoAlbums;}
    public Vector getMissingLocal()         {return this.missingLocal;}
    public Vector getMissingRemote()        {return this.missingRemote;}
    
}
