// Copyright (C) 2004, Brian Enigma <enigma at netninja.com>
// This file is part of iGallery.
//
// iGallery is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// iGallery is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Foobar; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
package org.ninjasoft.igallery.swingui;

import java.io.*;
import java.util.*;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.table.*;

import org.ninjasoft.igallery.logic.*;
import org.ninjasoft.igallery.data.*;

import com.apple.eawt.*;

public class MainFrame extends JFrame implements StatusListener {
    private boolean initialized = false;
    private Actions actions = new Actions();
    private JTextField urlText = new JTextField();
    private JTextField usernameText = new JTextField();
    private JPasswordField passwordText = new JPasswordField();
    private IPhotoTableModel iPhotoTableModel = new IPhotoTableModel();
    private GalleryTableModel galleryTableModel = new GalleryTableModel();
    private JTable iPhotoTable = new JTable(iPhotoTableModel);
    private JTable galleryTable = new JTable(galleryTableModel);
    private JButton loginButton = new JButton("Login");
    private JButton quitButton = new JButton("Quit");
    private JButton compareButton = new JButton("Compare");
    private JButton uploadButton = new JButton("Upload");
    private boolean loggedIn = false;
    private IGalleryLogic iGalleryLogic = new IGalleryLogic(this);
    private ApplicationProperties applicationProperties = null;
    private JTextArea status = new JTextArea();
    private String version = "";
    private JCheckBox debugCheckbox = new JCheckBox("Debug");

    public void initialize() {
        if (initialized)
            return;
        initialized = true;
        initializeApple();
        initializeVersion();
        initializeGui();
        initializeEvents();
        setLoggedIn(false);
        this.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
        iGalleryLogic.loadIPhotoAlbums();
        iPhotoTableModel.setAlbumList(iGalleryLogic.getIPhotoAlbums());
        initializePreferences();
    }
    
    private void initializeGui() {
        this.setSize(500, 400);
        this.setTitle("iGallery " + version);
        Dimension windowSize = this.getSize();
        Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
        this.setLocation(screenSize.width/2 - windowSize.width/2, screenSize.height/2 - windowSize.height/2);
        Font font = new Font("Dialog", Font.PLAIN, 9);
        status.setFont(font);
        status.setEditable(false);
        status.setOpaque(false);
        status.setRows(4);
        Container pane = this.getContentPane();
        pane.setLayout(new BorderLayout());
        pane.add(PanelBuilder.createLoginPanel(urlText, usernameText, passwordText, loginButton, debugCheckbox), BorderLayout.NORTH);
        pane.add(PanelBuilder.createLists(iPhotoTable, galleryTable), BorderLayout.CENTER);
        pane.add(PanelBuilder.createButtons(status, compareButton, uploadButton), BorderLayout.SOUTH);
        this.galleryTable.setDefaultRenderer(String.class, new DefaultTableCellRenderer() {
            public Component getTableCellRendererComponent(JTable table,  Object value,  boolean isSelected, boolean hasFocus,  int row,  int column) {
                Component result = super.getTableCellRendererComponent(table, value, isSelected, hasFocus, row, column);
                if (column == 1) {
                    ((DefaultTableCellRenderer) result).setHorizontalTextPosition(DefaultTableCellRenderer.RIGHT);
                    ((DefaultTableCellRenderer) result).setHorizontalAlignment(DefaultTableCellRenderer.RIGHT);
                }else{
                    ((DefaultTableCellRenderer) result).setHorizontalTextPosition(DefaultTableCellRenderer.LEFT);
                    ((DefaultTableCellRenderer) result).setHorizontalAlignment(DefaultTableCellRenderer.LEFT);
                }
                return result;
            }
        });
    }
    
    private void initializeEvents() {
        KeyListener keyListener = new KeyListener() {
            public void keyPressed(KeyEvent e) {
                int code = e.getKeyCode();
                int modifier = e.getModifiers();
                if (code == KeyEvent.VK_ENTER) {
                    toggleLoginState();
                    loginButton.requestFocus();
                } else if ((modifier == KeyEvent.META_MASK) && (code == KeyEvent.VK_W)) {
                    dispose();
                }
            }
            public void keyReleased(KeyEvent e) {}
            public void keyTyped(KeyEvent e) {}
        };
        urlText.addKeyListener(keyListener);
        usernameText.addKeyListener(keyListener);
        passwordText.addKeyListener(keyListener);
        this.loginButton.setActionCommand("login");
        this.loginButton.addActionListener(actions);
        this.compareButton.setActionCommand("compare");
        this.compareButton.addActionListener(actions);
        this.uploadButton.setActionCommand("upload");
        this.uploadButton.addActionListener(actions);
        debugCheckbox.addChangeListener(new ChangeListener() {
            public void stateChanged(ChangeEvent e) {
                Cookiejar.DEBUG = debugCheckbox.isSelected();
                GalleryConnection.DEBUG = debugCheckbox.isSelected();
            }
        });
    }
    
    private void initializePreferences() {
        try{
        	applicationProperties = new ApplicationProperties();
        }catch(java.io.IOException e){e.printStackTrace();}
        urlText.setText(applicationProperties.getProperty("url", "http://domain.com/gallery/gallery_remote2.php"));
        usernameText.setText(applicationProperties.getProperty("username", "admin"));
        passwordText.setText(applicationProperties.getProperty("password", ""));
        debugCheckbox.setSelected(applicationProperties.getProperty("debug", "false").equals("true"));
    }
    
    private void initializeApple() {
        System.setProperty("com.apple.mrj.application.apple.menu.about.name", "iGallery");
        System.setProperty("com.apple.laf.useScreenMenuBar", "true");
        System.setProperty("apple.awt.brushMetalLook", "true");
        Application application = Application.getApplication();
        application.addApplicationListener(new ApplicationAdapter() {
            public void handleQuit(ApplicationEvent event) {
                dispose();
            }
        });
        application.setEnabledAboutMenu(false);
        application.setEnabledPreferencesMenu(false);
    }
    
    private void initializeVersion() {
        InputStream in = this.getClass().getResourceAsStream("/org/ninjasoft/igallery/version.properties");
        if (in == null)
            return;
        Properties versionProperties = new Properties();
        try{
                versionProperties.load(in);
                in.close();
        }catch(Exception e) {}
        String versionTemp = versionProperties.getProperty("version", "");
        if ((versionTemp != null) && (versionTemp.length() > 0) && !versionTemp.equals("@VERSION@"))
            this.version = versionTemp;
    }
    
    private void setLoggedIn(boolean b) {
        this.loggedIn = b;
        urlText.setEnabled(!b);
        usernameText.setEnabled(!b);
        passwordText.setEnabled(!b);
        loginButton.setText(b ? "Logout" : "Login");
        uploadButton.setEnabled(b);
        quitButton.setEnabled(b);
        compareButton.setEnabled(b);
        galleryTable.setEnabled(b);
        iPhotoTable.setEnabled(b);
    }
    
    private void toggleLoginState() {
        if (this.loggedIn) {
            iGalleryLogic.logout();
            setLoggedIn(false);
        }else{
            loginButton.setEnabled(false);
            uploadButton.setEnabled(false);
            compareButton.setEnabled(false);
            if (iGalleryLogic.login(urlText.getText(), usernameText.getText(), passwordText.getText())) {
                setLoggedIn(true);
                this.galleryTableModel.setAlbumList(iGalleryLogic.getGalleryAlbums());
            }
            new Thread(new Runnable() {
                public void run() {
                    iGalleryLogic.getAlbumDetails();
                	loginButton.setEnabled(true);
                    uploadButton.setEnabled(true);
                    compareButton.setEnabled(true);
                }
            }).start();
        }
    }
    
    private void compare() {
        if (iPhotoTable.getSelectedRow() < 0)
            return;
        if (galleryTable.getSelectedRow() < 0)
            return;
        IPhotoAlbumData iPhotoAlbum = (IPhotoAlbumData) iGalleryLogic.getIPhotoAlbums().get(iPhotoTable.getSelectedRow());
        GalleryAlbum galleryAlbum = (GalleryAlbum) iGalleryLogic.getGalleryAlbums().get(galleryTable.getSelectedRow());
        iGalleryLogic.compareAlbums(iPhotoAlbum, galleryAlbum);
        int extraLocal = iGalleryLogic.getMissingRemote().size();
        int extraRemote = iGalleryLogic.getMissingLocal().size();
        String message = "";
        if ((extraLocal == 0) && (extraRemote == 0)) {
            message = "The albums are the same";
        }
        if (extraLocal > 0) {
            message = "There " +
            (extraLocal == 1 ? "is " : "are ") +
            + extraLocal + " picture" +
            (extraLocal == 1 ? "" : "s") +
            " in the iPhoto album but not yet on the server.";
        }
        if (extraRemote > 0) {
            if (message.length() > 0)
                message = message + "\n";
            message = message + "There " +
            (extraLocal == 1 ? "is " : "are ") +
            + extraRemote + " picture" +
            (extraLocal == 1 ? "" : "s") +
            " on the server but not in the iPhoto album.";
        }
        JOptionPane.showMessageDialog(this, message, "Info", JOptionPane.INFORMATION_MESSAGE);
    }
    
    private void upload() {
        new Thread(new Runnable() {
            public void run() {
                if (iPhotoTable.getSelectedRow() < 0)
                    return;
                if (galleryTable.getSelectedRow() < 0)
                    return;
                IPhotoAlbumData iPhotoAlbum = (IPhotoAlbumData) iGalleryLogic.getIPhotoAlbums().get(iPhotoTable.getSelectedRow());
                GalleryAlbum galleryAlbum = (GalleryAlbum) iGalleryLogic.getGalleryAlbums().get(galleryTable.getSelectedRow());
                iGalleryLogic.compareAlbums(iPhotoAlbum, galleryAlbum);
                Vector extraLocal = iGalleryLogic.getMissingRemote();
                int extraLocalSize = extraLocal.size();
                setStatus("" + extraLocalSize + " picture" + (extraLocalSize == 1 ? "" : "s") + " to upload");
                uploadButton.setEnabled(false);
                compareButton.setEnabled(false);
                loginButton.setEnabled(false);
                int counter = 1;
                int max = extraLocal.size();
                for (Iterator i = extraLocal.iterator(); i.hasNext(); ) {
                    IPhotoPictureData picture = (IPhotoPictureData) i.next();
                    setStatus("Uploading " + counter + " of " + max + ": " + picture.getCaption());
                    iGalleryLogic.upload(galleryAlbum, picture);
                    counter++;
                }
                iGalleryLogic.getAlbumDetails();
                uploadButton.setEnabled(true);
                compareButton.setEnabled(true);
                loginButton.setEnabled(true);
            }
        }).start();
    }
    
    public void setError(String message) {
        System.out.println(message);
        scrollStatus(message);
    }
    public void setStatus(String message) {
        int row = galleryTable.getSelectedRow();
        galleryTableModel.tellChanged();
        if (row > 0)
        	galleryTable.addRowSelectionInterval(row, row);
        System.out.println(message);
        scrollStatus(message);
    }
    public void scrollStatus(String message) {
        StringBuffer text = new StringBuffer(status.getText());
        text.append("\n");
        text.append(message);
        if (text.length() > 512) {
            int pos = text.indexOf("\n");
            text.delete(0, pos);
        }
        status.setText(text.toString());
        status.setCaretPosition(text.length());
    }
    
    public class Actions implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            String command = e.getActionCommand();
            command = command == null ? "" : command;
            if (command.equals("login"))
                toggleLoginState();
            else if (command.equals("compare"))
                compare();
            else if (command.equals("upload"))
                upload();
        
        }
    }

    public void dispose() {
        applicationProperties.setProperty("url", urlText.getText());
        applicationProperties.setProperty("username", usernameText.getText());
        applicationProperties.setProperty("password", passwordText.getText());
        applicationProperties.setProperty("debug", debugCheckbox.isSelected() ? "true" : "false");
        try{
        	applicationProperties.save();
        }catch(java.io.IOException e){e.printStackTrace();} 
        System.exit(0);
    }
    
    public void setVisible(boolean b) {
        initialize();
        super.setVisible(b);
    }

    
    public static void main(String[] args) {
        new MainFrame().setVisible(true);
    }
}
